@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2022 Intel Corporation
+Copyright (C) 2022-2024 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -24,7 +24,17 @@ inline llvm::Type *getNonOpaquePtrEltTy(const llvm::Type *PTy) {
 #endif
 }
 
+inline bool isOpaquePointerTy(const llvm::Type *Ty) {
+#if VC_INTR_LLVM_VERSION_MAJOR < 14
+  return false;
+#elif VC_INTR_LLVM_VERSION_MAJOR < 17
+  return Ty->isOpaquePointerTy();
+#else
+  return Ty->isPointerTy();
+#endif
+}
+
 } // namespace Type
 } // namespace VCINTR
 
-#endif // VCINTR_IR_TYPE_H
\ No newline at end of file
+#endif // VCINTR_IR_TYPE_H
